function [varargout]=extJinv(ls,S,J)
%%EXTJ Extensions for J levels of inverse SLWT
%   [L,R]=extJinv(LS,S,J) returns scalar values denoting left extension L
%   of segment starting with index S and right extension of segment ending
%   with index S-1 for J levels of inverse LWT with lifting scheme LS. 
%
%   [L,R,Ls,Rs]=extJinv(LS,S,J) in addition, the function returns vectors Ls,Rs 
%   containing all extensions 
%
%
%   See also: extJ   

%   Author(s): Z. Prusa
%   $Revision: 1.0 $  $Date: 2011/05/02$ 



left = 0;right = 0;

[leftSude,rightLiche] = extensions(ls,[0,1]);
[leftLiche,rightSude] = extensions(ls,[1,0]);

leftExtensions = [leftSude, leftLiche];
rightExtensions = [rightSude, rightLiche];

lastIndex = S-1;
firstIndex = S;

appLastInd = zeros(J+1,1);
appFirstInd = zeros(J+1,1);

rights = zeros(J,1);
lefts = zeros(J,1);

for j=0:J
  appLastInd(j+1)=floor(lastIndex/2^j); 
  appFirstInd(j+1)= appLastInd(j+1) + 1; 
end


helperIdx = 0;
for j=J:-1:1
   first = max([appFirstInd(j),helperIdx]);
   lj = leftExtensions(mod2(first)+1);
   lefts(j) = lj;
   appFirstInd(j) = first + lj;
   helperIdx = appFirstInd(j)*2;
end
left =  appFirstInd(j)-firstIndex;


helperIdx = Inf;
for j=J:-1:1
   last = min([appLastInd(j),helperIdx]);
   rj = rightExtensions(mod2(last)+1);
   rights(j) = rj;
   appLastInd(j) = last - rj;
   helperIdx = appLastInd(j)*2;
end

right = lastIndex-appLastInd(1);



varargout{1} = left;
varargout{2} = right;
varargout{3} = lefts;
varargout{4} = rights;



function y = mod2(x)
y = mod(x,2 );