function x = mylwt( x, LS, varargin)
%%LIFT2CELL Lifting wavelet transform
%   x = mylwt(x,LS) performs one level of lifting wavelet transform of
%   x using scheme LS.
%
%   x = mylwt(x,LS,J) performs J levels of lifting wavelet transform of
%   x using scheme LS.
%
%   x = mylwt(x,LS,J,S) performs J levels of lifting wavelet transform
%   of segment x strating at index S using scheme LS.
%
%   See also: myilwt   

%   Author(s): Z. Prusa
%   $Revision: 1.0 $  $Date: 2011/05/02$ 

firstIdx = 0;
if nargin<3
    order = 'LPfirst';
    level = 1;
end

if nargin==3
    order = 'LPfirst';
    level = varargin{1};
end

if nargin==4
    firstIdx = varargin{2};
    if(mod(varargin{2},2) == 0 ) order = 'LPfirst'; else order = 'HPfirst'; end
    level = varargin{1};
end

lx = length(x);

 firstIdxAPP = 1;
 firstIdxDET = 1+mod(firstIdxAPP,2);
shift = 0;

switch order
    case 'LPfirst'
        firstIdxAPP = 1;
        firstIdxDET = 1+mod(firstIdxAPP,2);
        shift = 0;
    case 'HPfirst'
        firstIdxAPP = 2;
        firstIdxDET = 1+mod(firstIdxAPP,2);
        shift = 1;
end

idxAPP = firstIdxAPP:2:lx;
idxDET = firstIdxDET:2:lx;
lenAPP = length(idxAPP);
lenDET = length(idxDET);

% Lifting.
NBL = size(LS,1);
LStype = LS{NBL,3};
for k = 1:NBL-1
    liftTYPE = LS{k,1};
    liftFILT = LS{k,2};
    DF       = LS{k,3};
    switch liftTYPE
       case 'p' ,  % primrn lifting (update) 
           x(idxAPP) = x(idxAPP) + ...
               mylsupdate(x(idxDET),liftFILT,DF,lenAPP,shift);
       case 'd' ,  % duln lifting (predict)
           x(idxDET) = x(idxDET) + ...
               mylspredict(x(idxAPP),liftFILT,DF,lenDET,shift);
    end
end


 if isempty(LStype)
     x(idxAPP) = LS{NBL,1}*x(idxAPP);
     x(idxDET) = LS{NBL,2}*x(idxDET);
 end

 if level>1
    x(idxAPP) = mylwt(x(idxAPP),LS,level-1,ceil(firstIdx/2));
 end



 
 



 
