 clc;clear all;close all;
 load schemes;
%%
% Script simulating real-time SLWT

% Segment length
lenx = 830;

% Signal data
x = randn(1,10*lenx+10)+2;
%x = 0:10*lenx-1+10;

% Variable to hold reconstructed data
xhat = zeros(size(x));

% Level of decomposition
J = 5;

% Lifting scheme
lscheme = cdf31;

% Vector to hold previous segment
xsminus1 = zeros(1,lenx);

% Initialization
ls = 0;  % left extension of the actual segment
lis = 0; % left extension of the actual segment for the inverse transform
Sis = 0; % first index in the actual segment for the inverse transform
LnorightSplus1 = 0; % left extension of the next segment
RskipS = 0; % number of samples which are left out at the end of the actual segment
overlap = []; % variable to hold overlap

% s index of segment, last segment (shorter than lenx) is omitted in this
% loop
for s=0:length(x)/lenx-1
    Ss = s*lenx; % starting index of the actual segment
    Ssplus1 = (s+1)*lenx; % starting index of the next segment
    [lsplus1, rs] = extJ(lscheme,Ssplus1,J); % right extension of the actual segment and left extension of the next segment
    RskipS = mod(2^J-mod(rs,2^J),2^J); % number of samples which are left out at the end of the actual segment

    xs = x(Ss+1:Ssplus1-RskipS); % picking actual segment from the input
    xsext = [xsminus1(end+1-LnorightSplus1:end),xs]; % extending the actual segment from the left using values from the saved previus segment in xsminus1
    Xtemp = mylwt(xsext,lscheme,J,Ss-LnorightSplus1); % performing transformation
    Xtemp = Xtemp(ls+1:end-rs); % cropping from both sides
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Xtemp contain coefficients to be processed
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
    LnorightSplus1 = lsplus1+ rs+ RskipS;  % left extension of the next segment
    Sisplus1 = Ssplus1 - (LnorightSplus1 - lsplus1); % starting index of the actual segment for the inverse transform
    [ris lisplus1] = extJinv(lscheme,Sisplus1,J); % right extension of the actual segment and left extension of the next segment for the inverse transform 
    Xtemp2 = [zeros(1,lis),Xtemp,zeros(1,ris)]; % extension of actual segment with zeros from both sides
    X1_hat = myilwt(Xtemp2,lscheme,J,Sis-lis); % performing inverse transform
    
    validLength = Sisplus1 - lisplus1 - (Sis - lis ); % number of valid samples from the beginning to be copied to the output
    overlapLength = lisplus1+ ris; % length of the overlap
    X1_hat(1:length(overlap))= X1_hat(1:length(overlap))+overlap(1:end); % adding overlap from previous segment
    xhat(Sis+1-lis:Sisplus1-lisplus1) = X1_hat(1:validLength); % copying valid samples to the output
    overlap = X1_hat(validLength+1:end); % saving overlap of the actual segment
    
    xsminus1 = x(Ss+1:Ssplus1); % save actual segment for naxt iteration
    ls = lsplus1;
    lis = lisplus1;
    Sis = Sisplus1;
end

% processing the last segment
Ss = Ssplus1;
Ssplus1 = length(x);
xs = x(Ss+1:Ssplus1);
xsext = [xsminus1(end+1-LnorightSplus1:end),xs];
Xtemp = mylwt(xsext,lscheme,J,Ss-LnorightSplus1);
Xtemp = Xtemp(ls+1:end);
Xtemp2 = [zeros(1,lis),Xtemp];
X1_hat = myilwt(Xtemp2,lscheme,J,Sis-lis);
X1_hat(1:min(end,length(overlap)))= X1_hat(1:min(end,length(overlap)))+overlap(1:min(end,length(X1_hat)));
xhat(Sis+1-lis:end) = X1_hat;

% plot the difference
stem([x'-xhat']);
